#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow),
    play_icon("play.png"),
    pause_icon("pause.png")
{
    capture = NULL;
    capture = cvCreateCameraCapture(-1);
    if(!capture) QMessageBox::critical(this, "Błąd krytyczny", "Nie wykryto kamery - nie można uruchomic programu");
    else cvReleaseCapture(&capture);
    QDir tmp_dir;
    tmp_dir.mkpath("Nagrania");
    ui->setupUi(this);
    start_signal = Phonon::createPlayer(Phonon::NoCategory, Phonon::MediaSource("./gong.wav"));
    ui->pb_play->setIcon(pause_icon);
    capture = cvCreateCameraCapture(0);
    in = cvQueryFrame(capture);
    out = cvCreateImage(cvSize(in->width, in->height), in->depth, in->nChannels);
    video = cvCreateImage(cvSize(in->width, in->height), in->depth, in->nChannels);
    gray1 = cvCreateImage(cvSize(in->width, in->height), in->depth, 1);
    gray2 = cvCreateImage(cvSize(in->width, in->height), in->depth, 1);
    foreground = cvCreateImage(cvSize(in->width, in->height), in->depth, 1);
    total_moves = cvCreateImage(cvSize(in->width, in->height), in->depth, 1);
    started = false;

    QTimer *timer = new QTimer();
    connect(timer, SIGNAL(timeout()), this, SLOT(timerAction()));
    timer->start(33);
    connect(ui->pb_start, SIGNAL(clicked()), this, SLOT(atStart()));
    connect(ui->pb_kamera, SIGNAL(clicked()), this, SLOT(setCamera()));
    connect(ui->pb_falstart, SIGNAL(clicked()), this, SLOT(atFalstart()));
    connect(ui->pb_play, SIGNAL(clicked()), this, SLOT(atPlay()));
    connect(ui->horizontalSlider, SIGNAL(actionTriggered(int)), this, SLOT(atSlider()));

    move(660, 120);
    round_time = 120;
    pause = false;
    checkingFalstart = false;
    writer = NULL;
    tempWriter = NULL;
    font = new CvFont;
    cvInitFont(font, CV_FONT_HERSHEY_TRIPLEX, 1.0, 1.0, 0.0);
    kamera = 0;
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::timerAction()
{
    int t = time.elapsed();
    if(t >= round_time*1000 && started)
    {
        started = false;
        ui->pb_start->setText("Start");
        if(writer) cvReleaseVideoWriter(&writer);
    }
    in = cvQueryFrame(capture);
    cvCopy(in, out);
    if(started)
    {
        cvCopy(in, video);
        if(!countdown)
        {
            display_time(t);
            std::string text = time2string(t);
            cvPutText(video, text.c_str(), cvPoint(10, 40), font, cvScalar(255, 255, 255, 0));
        }
        else
        {
            countdownFrames++;
            std::string text;
            if(t >= 5000 && started){
                display_time(0);
                text = time2string(0);
            }
            else{
                display_time(5000-t);
                text = time2string(5000-t);
            }
            cvPutText(video, text.c_str(), cvPoint(10, 40), font, cvScalar(0, 0, 255, 0));
            cvWriteFrame(tempWriter, video);
        }
        cvWriteFrame(writer, video);
    }
    if(t >= 2500 && t < 5000 && countdown)
    {
        if(!background)
        {
            cvCvtColor(in, gray1, CV_BGR2GRAY);
            cvSmooth(gray1, gray1);
            background = true;
        }
        cvCvtColor(in, gray2, CV_BGR2GRAY);
        cvSmooth(gray2, gray2);
        cvAbsDiff(gray1, gray2, foreground);
        cvThreshold(foreground, foreground, 15, 255, CV_THRESH_BINARY);
        cvErode(foreground, foreground);
        for(int y = 0; y < foreground->height; y++)
        {
            for(int x = 0; x < foreground->width; x++)
            {
                uchar* mask_ptr = (uchar*)(foreground->imageData + y*foreground->widthStep + x);
                if(*mask_ptr == 255)
                {
                    uchar* ptr = (uchar*)(out->imageData + y*out->widthStep + x*3);
                    ptr[2] = 255;
                }
            }
        }
        cvOr(foreground, total_moves, total_moves);
    }
    if(t >= 5000 && started)
    {
        if(countdown)
        {
            display_time(0);
            ui->pb_start->setEnabled(true);
            ui->pb_start->setText("OK");
            ui->pb_falstart->setEnabled(true);
            for(int y = 0; y < total_moves->height; y++)
            {
                for(int x = 0; x < total_moves->width; x++)
                {
                    uchar* mask_ptr = (uchar*)(total_moves->imageData + y*total_moves->widthStep + x);
                    if(*mask_ptr == 255)
                    {
                        uchar* ptr = (uchar*)(out->imageData + y*out->widthStep + x*3);
                        ptr[2] = 255;
                    }
                }
            }
            countdown = false;
            ui->horizontalSlider->setMaximum(countdownFrames);
            if(tempWriter) cvReleaseVideoWriter(&tempWriter);
            time.restart();
            pause = true;
            cvShowImage("Minisumo", out);
        }
    }
    if(!pause)
        cvShowImage("Minisumo", out);
    if(checkingFalstart)
    {
        if(!CFPaused){
            CFIn = cvQueryFrame(falstartCapture);
            ui->horizontalSlider->setValue(ui->horizontalSlider->value()+1);
        }
        if(CFIn) cvShowImage("Falstart", CFIn);
    }
}

void MainWindow::atStart()
{
    if(pause)
    {
        pause = false;
        ui->pb_start->setText("Stop");
        ui->pb_falstart->setEnabled(false);
        if(checkingFalstart){
            checkingFalstart = false;
            cvDestroyWindow("Falstart");
            ui->pb_falstart->setText("Sprawdź");
            ui->horizontalSlider->setEnabled(false);
            ui->pb_play->setEnabled(false);
            CFPaused = true;
            atPlay();
        }
    }
    else if (!started)
    {
        countdownFrames = 0;
        start_signal->play();
        ui->horizontalSlider->setValue(0);
        QString nazwa = "Nagrania/"+ui->le_nazwa->text() + ".avi";
        ui->le_nazwa->setEnabled(false);
        writer = cvCreateVideoWriter(nazwa.toAscii(), CV_FOURCC('M','J', 'P','G'), 30, cvSize(in->width, in->height));
        tempWriter = cvCreateVideoWriter("temp.avi", CV_FOURCC('M','J', 'P','G'), 30, cvSize(in->width, in->height));
        ui->pb_start->setDisabled(true);
        started = true;
        background = false;
        countdown = true;
        time.start();
        cvZero(total_moves);
    }
    else
    {
        ui->pb_start->setText("Start");
        ui->pb_falstart->setEnabled(false);
        started = false;
        if(writer) cvReleaseVideoWriter(&writer);
        ui->le_nazwa->setEnabled(true);
    }
}

void MainWindow::display_time(int time)
{
    ui->lcd_time->display((time%600000)/60000);
    ui->lcd_time_2->display((time%60000)/10000);
    ui->lcd_time_3->display((time%10000)/1000);
    ui->lcd_time_4->display((time%1000)/100);
    ui->lcd_time_5->display((time%100)/10);
}

void MainWindow::at2minutes()
{
    round_time = 120;
}

void MainWindow::at3minutes()
{
    round_time = 180;
}

std::string MainWindow::time2string(int t)
{
    std::string text = "";
    text += ((t%600000)/60000)+'0';
    text += ':';
    text += ((t%60000)/10000)+'0';
    text += ((t%10000)/1000)+'0';
    text += ':';
    text += ((t%1000)/100)+'0';
    text += ((t%100)/10)+'0';
    return text;
}

void MainWindow::setCamera()
{
    kamera++;
    if (kamera > 3) kamera = 0;
    if (kamera == 0)
        ui->pb_kamera->setText("Kamera 0");
    if (kamera == 1)
        ui->pb_kamera->setText("Kamera 1");
    if (kamera == 2)
        ui->pb_kamera->setText("Kamera 2");
    if (kamera == 3)
        ui->pb_kamera->setText("Kamera 3");
    cvReleaseCapture(&capture);
    capture = cvCreateCameraCapture(kamera);
    if(!capture) capture = cvCreateCameraCapture(0);
    if(!capture) std::cerr<<"Nie podlaczono kamery"<<std::endl;
}

void MainWindow::atFalstart()
{
    if(checkingFalstart){
        checkingFalstart = false;
        cvDestroyWindow("Falstart");
        ui->pb_falstart->setText("Sprawdź");
        ui->pb_start->setText("Start");
        ui->pb_falstart->setEnabled(false);
        started = false;
        if(writer) cvReleaseVideoWriter(&writer);
        ui->le_nazwa->setEnabled(true);
        ui->horizontalSlider->setEnabled(false);
        ui->pb_play->setEnabled(false);
        pause = false;
        CFPaused = true;
        atPlay();
    }
    else{
        cvNamedWindow("Falstart");
        checkingFalstart = true;
        CFPaused = false;
        falstartCapture = cvCreateFileCapture("temp.avi");
        ui->pb_falstart->setText("Falstart");
        ui->horizontalSlider->setEnabled(true);
        ui->pb_play->setEnabled(true);
    }
}

void MainWindow::atPlay()
{
    if(CFPaused){
        CFPaused = false;
        ui->pb_play->setIcon(pause_icon);
    }
    else{
        CFPaused = true;
        ui->pb_play->setIcon(play_icon);
    }
}

void MainWindow::atSlider()
{
    cvSetCaptureProperty(falstartCapture, CV_CAP_PROP_POS_FRAMES, ui->horizontalSlider->value());
    CFIn = cvQueryFrame(falstartCapture);
    cvShowImage("Falstart", CFIn);
}
